import urllib

import bs4
import requests

from .exceptions import *


class MusicalMusic:
    """Musescore actions requiring an account."""

    def __init__(self, username, password):

        url = "https://musescore.com/user/login"
        r = requests.get(url)
        soup = bs4.BeautifulSoup(r.text, "html.parser")
        csrf = soup.find("meta", {"name": "csrf-token"})["content"]
        url = "https://musescore.com/user/auth/login/process"
        cookies = {
            "mu_browser_uni": r.cookies['mu_browser_uni'],
            "_csrf": r.cookies["_csrf"]
        }
        data = {
          "username": username,
          "password": password,
          "_csrf": csrf,
          "op": "Log in"
        }
        try:
            mu_user = requests.post(url,
                                    data=data,
                                    cookies=cookies,
                                    allow_redirects=False,
                                    ).cookies["mu_user_new"]
        except KeyError as e:
            raise InvalidCredentials(
                "Please check your username and password!") from e

        mu_browser_uni = r.cookies['mu_browser_uni']

        self.mu_browser_uni = mu_browser_uni
        self.mu_user = mu_user

    def retrieve(self, id, format="pdf"):
        """Retrieves Musescore data in bytes"""

        if format not in ["mp3", "pdf", "mid", "mxl", "mscz"]:
            raise InvalidFileExtension("Must be mp3, pdf, mid, mxl, or mscz.")
        newlink = f"https://musescore.com/score/{id}/download/{format}"
        cookies = {"mu_browser_uni": self.mu_browser_uni,
                   "mu_user_new": self.mu_user}
        bytes = requests.get(newlink, cookies=cookies, verify=False)
        if bytes.status_code != 200:
            raise InvalidScoreID(str(bytes.status_code))
        return bytes.content

    def download(self, id, filename, format="mp3"):
        if format not in ["mp3", "pdf", "mid", "mxl", "mscz"]:
            raise InvalidFileExtension("Must be mp3, pdf, mid, mxl, or mscz.")
        newlink = f"https://musescore.com/score/{id}/download/{format}"
        opener = urllib.request.build_opener()
        cookieString = f"mu_browser_uni={self.mu_browser_uni};" \
                        f"mu_user_new={self.mu_user}"
        opener.addheaders = [("cookie", cookieString)]
        urllib.request.install_opener(opener)
        try:
            urllib.request.urlretrieve(newlink, filename)
        except urllib.error.HTTPError:
            raise InvalidScoreID("The ID of the score is invalid!")




class MuseScoreException(Exception):
    pass

class InvalidFileExtension(MuseScoreException):
    pass

class InvalidScoreID(MuseScoreException):
    pass

class InvalidCredentials(MuseScoreException):
    pass

class InvalidSearchSort(MuseScoreException):
    pass



from setuptools import setup, find_packages

setup(
  name="MusicalMusic",
  version="0.2.2",
  description="Retrieve Sheet music from Musescore!",
  long_description=open("README.md").read(),
  long_description_content_type='text/markdown',
  url="https://github.com/frankye8998/MusicalMusic/",
  author="Frank Ye",
  author_email="frankfrankfrankyeyeye@gmail.com",
  license="GPL 3.0",
  classifiers=[
    "Programming Language :: Python :: 3"
  ],
  packages=find_packages(),
  install_requires=["requests","bs4"]
)




import requests
import bs4
import urllib
from .exceptions import *


def search(song, sort="relevance"):
    """Static search function"""
    if sort not in ["relevance", "date_uploaded", "comment_count", "view_count"]:
        raise InvalidSearchSort("Results must be sorted by either \"relevance\", \"date_uploaded\", \"comment_count\", or \"view_count\"")
    query = {"text": song, "sort": sort}
    search = requests.get(f"https://musescore.com/sheetmusic?"
                          f"{urllib.parse.urlencode(query)}").text
    soup = bs4.BeautifulSoup(search, "html.parser")
    listoftitles = []
    for i in soup.findAll("div", {"class": "col-right"}):
        listoftitles.append({
            "id": i.find("a")["href"].split("/")[-1],
            "title": i.find("a").text.strip(),
            "instruments": i.find("div", {"class": "instruments"}).text,
            "duration": i.findAll("span")[2].text,
            "pages": int(i.findAll("span")[1].text.split(" ")[0]),
            "views": int(i.findAll("span")[4].text.split(" ")[0]
                         .replace(",", "")),
            "author": {
                "username": i.find("article").text.split("\n")[1],
                "user_id": i.find("article").find("a")["href"].split("/")[-1]
            },
            "desc": i.find("div", {"class": "expander"}).text}
        )
    return listoftitles
